import json
import secrets
from typing import Dict, Tuple

import redis

from quiz_bots import settings


def connect_to_redis() -> redis.client.Redis:
    """Creates a connection to the Redis database."""
    redis_host, redis_port = settings.db_endpoint.split(':')
    redis_password = settings.db_password
    return redis.Redis(
        host=redis_host,
        port=redis_port,
        password=redis_password,
    )


def get_quiz_content_from_redis(redis_connection: redis.client.Redis):
    """Gets the content for the quiz from the database."""
    quiz_content = redis_connection.get('questions')
    if quiz_content:
        return json.loads(quiz_content)


def get_random_question(
    quiz_content: Dict[str, Dict[str, str]],
) -> Tuple[Dict[str, str], int]:
    """Gets a random question from the quiz content."""
    number_of_question = secrets.randbelow(len(quiz_content) + 1)
    return quiz_content[f'question_{number_of_question}'], number_of_question


def save_last_asked_question_to_redis(
    user_id, redis_connection, social_network: str, number_of_question: int,
) -> None:
    """Saves the user's question number in the database."""
    user = {
        f'user_{social_network}_{user_id}': {
            'last_asked_question': f'question_{number_of_question}',
        },
    }
    redis_connection.set('users', json.dumps(user))


def get_last_asked_question_from_redis(
    user_id, redis_connection, social_network: str,
):
    """Gets the last asked question from redis."""
    users_data = json.loads(redis_connection.get('users'))
    user_data = users_data[f'user_{social_network}_{user_id}']
    return user_data['last_asked_question']


def get_answer_to_last_question_of_user(
    redis_connection, last_asked_question: Dict[str, str],
) -> str:
    """Gets the text of the answer to the last question asked by the user."""
    quiz_content = json.loads(redis_connection.get('questions'))
    return quiz_content[last_asked_question]['answer']



from telegram.ext import (
    CommandHandler,
    ConversationHandler,
    Filters,
    MessageHandler,
    Updater,
)

from quiz_bots import settings
from quiz_bots.bots_helpers import send_message_to_tg_chat
from quiz_bots.quiz_helpers import get_clean_text_of_answer
from quiz_bots.redis_utils import (
    connect_to_redis,
    get_answer_to_last_question_of_user,
    get_last_asked_question_from_redis,
    get_quiz_content_from_redis,
    get_random_question,
    save_last_asked_question_to_redis,
)

REDIS_CONNECTION = connect_to_redis()
QUIZ_CONTENT = get_quiz_content_from_redis(REDIS_CONNECTION)

BUTTON_NEW_QUESTION = 'Новый вопрос'
BUTTON_SURRENDER = 'Сдаться'
BUTTON_SCORE = 'Мой счет'
QUESTION_IS_SET_INCORRECTLY_BUTTON = 'Вопрос составлен неверно'

CHOOSING, WAIT_ANSWER = range(2)


def welcome_message(update, context):
    """Sends a welcome message to the chat when the bot starts."""
    first_name = update.message.from_user.first_name
    message_text = f'Приветствую, {first_name}! Для старта нажмите кнопку "Новый вопрос"'
    custom_keyboard = [[BUTTON_NEW_QUESTION]]
    send_message_to_tg_chat(
        update,
        message_text,
        custom_keyboard,
    )

    return CHOOSING


def handle_new_question_request(update, context) -> int:
    """Sends a random question to the user."""
    question, number_of_question = get_random_question(QUIZ_CONTENT)

    if context.user_data.get('was_the_surrender_button_pressed'):
        custom_keyboard = [
            [BUTTON_SURRENDER, BUTTON_SCORE],
            [QUESTION_IS_SET_INCORRECTLY_BUTTON],
        ]
    else:
        custom_keyboard = [
            [BUTTON_SURRENDER, BUTTON_SCORE],
        ]
    send_message_to_tg_chat(
        update,
        question['question'].replace('\n', ' '),
        custom_keyboard,
    )

    save_last_asked_question_to_redis(
        user_id=update.message.from_user.id,
        redis_connection=REDIS_CONNECTION,
        social_network='tg',
        number_of_question=number_of_question,
    )

    return WAIT_ANSWER


def handle_solution_attempt(update, context):
    """Checks answer of user."""
    last_asked_question = get_last_asked_question_from_redis(
        user_id=update.message.from_user.id,
        redis_connection=REDIS_CONNECTION,
        social_network='tg',
    )
    answer_to_last_question_of_user = get_answer_to_last_question_of_user(
        redis_connection=REDIS_CONNECTION,
        last_asked_question=last_asked_question,
    )
    clean_answer = get_clean_text_of_answer(answer_to_last_question_of_user)
    if update.message.text.lower() == clean_answer:
        message_text = 'Правильно! Для следующего вопроса нажми «Новый вопрос»'
        custom_keyboard = [[BUTTON_NEW_QUESTION]]
    else:
        message_text = 'Неправильно... Попробуешь ещё раз?'
        custom_keyboard = [[BUTTON_SURRENDER, BUTTON_SCORE]]

    send_message_to_tg_chat(
        update,
        message_text,
        custom_keyboard,
    )

    return CHOOSING


def handle_opportunity_to_surrender(update, context):
    """Handles clicking the "Surrender" button.

    The bot sends the user an answer to the question
    and sends the next question with the next message.
    """
    last_asked_question = get_last_asked_question_from_redis(
        user_id=update.message.from_user.id,
        redis_connection=REDIS_CONNECTION,
        social_network='tg',
    )
    answer_to_last_question_of_user = get_answer_to_last_question_of_user(
        redis_connection=REDIS_CONNECTION,
        last_asked_question=last_asked_question,
    )
    send_message_to_tg_chat(
        update,
        answer_to_last_question_of_user,
    )
    context.user_data['was_the_surrender_button_pressed'] = True

    handle_new_question_request(update, context)

    return WAIT_ANSWER


def handler_question_is_incorrect(update, context):  # noqa: WPS226
    """Handles clicking the button 'The question is incorrect'."""
    custom_keyboard = [[BUTTON_SURRENDER, BUTTON_SCORE]]
    send_message_to_tg_chat(
        update,
        'Спасибо за фидбек! Ждем ответ на предыдущий вопрос!',
        custom_keyboard,
    )
    return WAIT_ANSWER


def cancel(update, context):
    """Sends a message to the chat when the bot finish work."""
    first_name = update.message.from_user.first_name
    message_text = f'Спасибо за участие в викторине, {first_name}!'
    custom_keyboard = [[BUTTON_NEW_QUESTION]]
    send_message_to_tg_chat(
        update,
        message_text,
        custom_keyboard,
    )
    return ConversationHandler.END


def start_tg_bot():
    """Launch the TG bot."""
    updater = Updater(settings.tg_token, use_context=True)

    dp = updater.dispatcher

    conv_handler = ConversationHandler(
        entry_points=[CommandHandler('start', welcome_message)],

        states={
            CHOOSING: [
                MessageHandler(
                    Filters.regex('^Новый вопрос$'),
                    handle_new_question_request,
                ),
                MessageHandler(
                    Filters.regex('^Сдаться$'),
                    handle_opportunity_to_surrender,
                ),
            ],
            WAIT_ANSWER: [
                MessageHandler(
                    Filters.regex('^Сдаться$'),
                    handle_opportunity_to_surrender,
                ),
                MessageHandler(
                    Filters.regex('^Вопрос составлен неверно$'),
                    handler_question_is_incorrect,
                ),
                MessageHandler(
                    Filters.text,
                    handle_solution_attempt,
                ),
            ],
        },

        fallbacks=[CommandHandler('cancel', cancel)],
    )

    dp.add_handler(conv_handler)

    updater.start_polling()
    updater.idle()



import string


def get_clean_text_of_answer(answer_text: str) -> str:
    """Gets the question text cleared of punctuation marks."""
    clean_text_of_answer = answer_text.translate(
        str.maketrans(
            '',
            '',
            string.punctuation,
        ),
    )
    return clean_text_of_answer.strip().lower()



from environs import Env

env = Env()
env.read_env()

tg_token = env.str('TELEGRAM_TOKEN')
db_endpoint = env.str('DB_ENDPOINT')
db_password = env.str('DB_PASSWORD')
vk_group_token = env.str('VK_GROUP_TOKEN')



import argparse

from quiz_bots.quiz import send_quiz_content_to_redis
from quiz_bots.tg_bot import start_tg_bot
from quiz_bots.vk_bot import start_vk_bot


def get_args() -> argparse.Namespace:
    """Getting arguments from command line."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--start_bot',
        type=str,
        choices=['vk', 'tg'],
        help='Which bot to launch?',
    )

    parser.add_argument(
        '--export_quiz_content_to_redis',
        type=str,
        help='Specify path to folder with quiz questions files',
    )
    return parser.parse_args()


def main() -> None:
    """Entry point."""
    args = get_args()
    start_bot = args.start_bot
    quiz_questions_folder_path = args.export_quiz_content_to_redis

    if start_bot == 'tg':
        start_tg_bot()
    elif start_bot == 'vk':
        start_vk_bot()

    if quiz_questions_folder_path:
        send_quiz_content_to_redis(quiz_questions_folder_path)


if __name__ == '__main__':
    main()



import glob
import json
import re
from dataclasses import asdict, dataclass
from typing import Dict, Generator, List

from quiz_bots.redis_utils import connect_to_redis

REDIS_CONNECTION = connect_to_redis()


@dataclass
class QuizRound:
    """Contains the data for the quiz round."""

    question: str
    answer: str


def get_raw_quiz_content_from_file(path_to_file: str) -> str:
    """Retrieves raw content for the quiz from a file for further processing."""
    with open(path_to_file, 'r', encoding='KOI8-R') as file_object:
        return file_object.read()


def fetch_quiz_data_from_text_block(
    text_block: str, text_blocks: Generator[str, None, None],
) -> QuizRound:
    """Fetches query data from a text block."""
    return QuizRound(
        question=re.sub(r'Вопрос \d+:\n', '', text_block),
        answer=re.sub(r'Ответ:\n', '', next(text_blocks)),
    )


def prepare_quiz_content_from(raw_quiz_content: str) -> List[QuizRound]:
    """Prepares the content for the quiz from the raw data."""
    text_blocks = (text_block for text_block in raw_quiz_content.split('\n\n'))
    return [
        fetch_quiz_data_from_text_block(text_block, text_blocks)
        for text_block in text_blocks
        if text_block.startswith('Вопрос')
    ]


def get_all_quiz_content(file_paths: List[str]) -> List[QuizRound]:
    """Getting quiz content from all files in folder."""
    all_quiz_content = []
    for file_path in file_paths:
        raw_quiz_content = get_raw_quiz_content_from_file(file_path)
        quiz_content = prepare_quiz_content_from(raw_quiz_content)
        all_quiz_content.extend(quiz_content)
    return all_quiz_content


def prepare_data_for_sending_to_redis(
    all_quiz_content: List[QuizRound],
) -> Dict[str, Dict[str, str]]:
    """Convert quiz content to dict for sending to Redis."""
    return {
        f'question_{question_count + 1}': asdict(question_data)
        for question_count, question_data in enumerate(all_quiz_content)
    }


def send_quiz_content_to_redis(quiz_folder_path: str) -> None:
    """Sends quiz content to Redis."""
    file_paths = list(glob.glob(f'{quiz_folder_path}/**/*.txt', recursive=True))
    all_quiz_content = get_all_quiz_content(file_paths)
    quiz_content_dict = prepare_data_for_sending_to_redis(all_quiz_content)
    REDIS_CONNECTION.set('questions', json.dumps(quiz_content_dict))
